#!/usr/bin/perl
#
# Copyright (C) 2015 Roland Laifer 
#                    Karlsruhe Institute of Technology (KIT), 
#                    Steinbuch Centre for Computing (SCC)
#                    76128 Karlsruhe, Germany
#                    Roland.Laifer (at) kit.edu
#
# ------------------------------------------------------------------------
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ------------------------------------------------------------------------
#
# $Log: compare_user_group_ost_quotas.pl,v $
# Revision 1.1  2015/12/23 14:05:01  rz54
# Initial revision
#
#
# $Id: compare_user_group_ost_quotas.pl,v 1.1 2015/12/23 14:05:01 rz54 Exp $

use strict;
use warnings;

use Getopt::Std;
our($opt_d, $opt_p, $opt_h, $opt_v);

require "getopts.pl";

(my $myname = $0) =~ s#^.*/##;

sub usage {
    print "$myname checks for a given OST if sum of user and group quotas are similar.\n";
    print "Usage: $myname [-h] -d <FS-OST>\n";
    print "       -d <FS-OST>     file system and OST name (example: pfscwor1-OST0000)\n";
    print "       -p <number>     report warning if difference in per thousand is above (default: 10)\n";
    print "       -h              this help\n";
    print "       -v              verbose\n";
    print <<'EOF';
Example with pdsh:
pdsh -f1 -w server[3-4] 'for ost in $(ls /proc/fs/lustre/osd-ldiskfs/ | grep OST); do echo $ost:; ./bin/compare_user_group_ost_quotas.pl -d $ost -p1 -v; done'
EOF
    exit 0;
}

# get options
getopts("d:p:hv") || &usage;
usage if ( $opt_h );
usage unless ( $opt_d );
usage unless ($#ARGV == -1);
my $diffper1000;
if ( $opt_p ) {
    $diffper1000 = $opt_p;
} else {
    $diffper1000 = 10;
}

my $userinodes = 0;
my $userkbytes = 0;
my $cmd="cat /proc/fs/lustre/osd-ldiskfs/${opt_d}/quota_slave/acct_user";
open my $cat_proc_h, "-|", $cmd or die "*** Command $cmd failed: $!\n";
while (<$cat_proc_h>) {
    if ( /usage:.*inodes:\s+(\d+),\s+kbytes:\s+(\d+)\s+/ ) {
	$userinodes += $1;
	$userkbytes += $2;
    }
}

my $groupinodes = 0;
my $groupkbytes = 0;
$cmd="cat /proc/fs/lustre/osd-ldiskfs/${opt_d}/quota_slave/acct_group";
open $cat_proc_h, "-|", $cmd or die "*** Command $cmd failed: $!\n";
while (<$cat_proc_h>) {
    if ( /usage:.*inodes:\s+(\d+),\s+kbytes:\s+(\d+)\s+/ ) {
	$groupinodes += $1;
	$groupkbytes += $2;
    }
}

printf ("Sum of inodes of users: %12d\n", $userinodes) if ($opt_v);
printf ("Sum of inodes of groups: %11d\n", $groupinodes) if ($opt_v);
printf ("Sum of kbytes of users: %12d\n", $userkbytes) if ($opt_v);
printf ("Sum of kbytes of groups: %11d\n", $groupkbytes) if ($opt_v);

if ($groupinodes > 0) {
    my $per1000 = ($userinodes / $groupinodes) * 1000;
    if ($per1000 < (1000 - $diffper1000) || $per1000 > (1000 + $diffper1000)) {
	printf ("!!! Warning: Difference between user and group inodes is higher than %2d per mil (%3.2f)!\n", $diffper1000, $per1000);
    }
}
if ($groupkbytes > 0) {
    my $per1000 = ($userkbytes / $groupkbytes) * 1000;
    if ($per1000 < (1000 - $diffper1000) || $per1000 > (1000 + $diffper1000)) {
	printf ("!!! Warning: Difference between user and group kbytes is higher than %2d per mil (%3.2f)!\n", $diffper1000, $per1000);
    }
}
